@@ -1,11 +1,11 @@
 from ast import literal_eval as _literal_eval
 from datetime import datetime as _datetime
 from functools import partial as _partial
-from json import load as _jload
 from logging import warning as _warning
 from pathlib import Path
+from re import fullmatch as _fullmatch
 
-from pandas import to_datetime as _to_datetime
+from pandas import to_datetime as _to_datetime, read_csv as _read_csv
 
 from . import _FARSI_NORM, _MarketState, _api, _csv2df, _F, _TypedDict, \
     _get_data, \
@@ -16,7 +16,7 @@ from . import _FARSI_NORM, _MarketState, _api, _csv2df, _F, _TypedDict, \
 
 _strptime = _datetime.strptime
 _j_ymd_parse = _partial(_jstrptime, format='%Y/%m/%d')
-_DB_PATH = Path(__file__).parent / 'database/ids.json'
+_DS_PATH = Path(__file__).parent / 'dataset/dataset.csv'
 
 
 _FARSI_NORM_REVERSED = {v: k for k, v in _FARSI_NORM.items()}
@@ -65,17 +65,19 @@ _RELATED_COMPANIES = _rc(r"var RelatedCompanies=(\[.*\]);").search
 _TRADE_HISTORY = _rc(r"var TradeHistory=(\[.*\]);").search
 _STR_TO_NUM = _partial(_rc(rf"'{_F}'").sub, r'\1')
 
-with open(_DB_PATH, encoding='utf8') as _f:
-    _L18S: dict[str, tuple] = _jload(_f)
 
-_INS_CODE_TO_L18 = None
+_DS = _read_csv(_DS_PATH, low_memory=False, lineterminator='\n')
+_L18S = {k: v for k, (*v,) in zip(_DS['l18'], _DS.itertuples(index=False))}
+del _DS  # don't need this anymore
+
+_CODE_TO_L18 = None
 
 
-def _l18_l30(ins_code: int) -> tuple:
-    global _INS_CODE_TO_L18
-    if _INS_CODE_TO_L18 is None:
-        _INS_CODE_TO_L18 = {v[0]: k for k, v in _L18S.items()}
-    return _L18S[_INS_CODE_TO_L18[ins_code]][1:]
+def _l18_l30(code: int) -> tuple:
+    global _CODE_TO_L18
+    if _CODE_TO_L18 is None:
+        _CODE_TO_L18 = {v[0]: k for k, v in _L18S.items()}
+    return _L18S[_CODE_TO_L18[code]][1:]
 
 
 class _IntraDay(_TypedDict, total=False):
@@ -696,18 +698,24 @@ async def search(skey: str, /) -> _DataFrame:
     return _csv2df(
         _StringIO(await _get_data('search.aspx?skey=' + skey, fa=True)),
         header=None,
+        # Another terminology would be: {
+        #   ins_code: round_lot,
+        #   retail: odd_lot,
+        #   compensation: buyback,
+        #   wholesale: block,
+        # } see: https://www.sena.ir/news/77488/
         names=(
             'l18', 'l30', 'ins_code', 'retail', 'compensation', 'wholesale',
             '_unknown1', '_unknown2', '_unknown3', '_unknown4', '_unknown5'))
 
 
 def _parse_price_info(price_info):
-    price_info = price_info.split(',')
+    price_parts = price_info.split(',')
 
-    if len(price_info) == 17:
+    if len(price_parts) == 17:
         # this is unexpected according to js source
-        assert price_info[14] == price_info[10]
-        del price_info[14]
+        assert price_parts[14] == price_parts[10]
+        del price_parts[14]
 
     (  # see dev/tsetmc_source_files/Instinfo.js
         timestamp,  # 0
@@ -726,7 +734,7 @@ def _parse_price_info(price_info):
         last_info_time,  # 13
         nav_datetime,  # 14
         nav  # 15
-    ) = price_info
+    ) = price_parts
 
     result = {
         'timestamp': timestamp, 'status': status
@@ -738,6 +746,11 @@ def _parse_price_info(price_info):
 
     if nav:
         result['nav'] = int(nav)
-        result['nav_datetime'] = _jstrptime(nav_datetime, '%Y/%m/%d %H:%M:%S')
+        try:
+            _jdatetime(1400, 12, 1, 16, 30, 0)
+            _fullmatch(r'(\d+)/(\d+)/(\d+) (\d+):(\d+):(\d+)', nav_datetime)
+            result['nav_datetime'] = _jstrptime(nav_datetime, '%Y/%m/%d %H:%M:%S')
+        except ValueError:  # could be invalid day of the month
+            result['nav_datetime'] = nav_datetime
 
     return result
